"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var classnames_1 = require("classnames");
var message_box_1 = require("../message-box");
var icon_1 = require("../icon");
var clsPrefix = 'next-notification';
var placements = {
    'topLeft': 'top-left',
    'topRight': 'top-right',
    'bottomLeft': 'bottom-left',
    'bottomRight': 'bottom-right',
};
var instances = {};
var notice = function (_a) {
    var _b = _a.duration, duration = _b === void 0 ? 4 : _b, placement = _a.placement, title = _a.title, description = _a.description, onClose = _a.onClose, icon = _a.icon;
    var _c, _d;
    var instance = getMessageBoxInstance(placements[placement]);
    var notificationIcon = null;
    var noticeClasses = classnames_1.default((_c = {},
        _c[clsPrefix + "-message"] = true,
        _c[clsPrefix + "-with-icon"] = !!icon,
        _c[clsPrefix + "-with-desc"] = description,
        _c));
    var iconType = {
        'success': ['checkmark-circle', 'checkmark-circle-o'],
        'error': ['close-circle', 'close-circle-o'],
        'info': ['information-circle', 'information-circle-o'],
        'warning': ['warning', 'warning'],
    }[icon];
    if (typeof icon === 'string') {
        if (iconType) {
            if (description) {
                notificationIcon = React.createElement(icon_1.default, { type: iconType[1] });
            }
            else {
                notificationIcon = React.createElement(icon_1.default, { type: iconType[0] });
            }
        }
        else {
            notificationIcon = React.createElement(icon_1.default, { type: icon });
        }
    }
    else {
        notificationIcon = icon;
    }
    var iconClasses = classnames_1.default((_d = {},
        _d[clsPrefix + "-icon"] = notificationIcon,
        _d[clsPrefix + "-icon-" + icon] = !!iconType,
        _d));
    instance.showMessage({
        duration: duration,
        content: (React.createElement("div", { className: noticeClasses },
            notificationIcon ? (React.createElement("span", { className: iconClasses }, notificationIcon)) : null,
            React.createElement("div", { className: clsPrefix + '-title' }, title),
            description ? (React.createElement("div", { className: clsPrefix + '-description' }, description)) : null,
            React.createElement("span", { className: clsPrefix + '-close' },
                React.createElement(icon_1.default, { type: 'close' })))),
        onClose: onClose
    });
};
var getMessageBoxInstance = function (placement) {
    if (placement === void 0) { placement = 'topRight'; }
    var instance = instances[placement] || message_box_1.default.newInstance({
        clsPrefix: clsPrefix,
        style: { right: 12, top: 24 }
    });
    instances[placement] = instance;
    return instance;
};
exports.default = {
    open: function (config) {
        notice(config);
    },
    info: function (config) {
        var mergedConfig = Object.assign(config, { icon: 'info' });
        notice(mergedConfig);
    },
    success: function (config) {
        var mergedConfig = Object.assign(config, { icon: 'success' });
        notice(mergedConfig);
    },
    warning: function (config) {
        var mergedConfig = Object.assign(config, { icon: 'warning' });
        notice(mergedConfig);
    },
    error: function (config) {
        var mergedConfig = Object.assign(config, { icon: 'error' });
        notice(mergedConfig);
    }
};
