"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var classnames_1 = require("classnames");
var icon_1 = require("../icon");
var clsPrefix = 'next-select';
var defaultFilter = function (inputValue, option) {
    var label = option.label;
    return typeof label === 'string' && label.toLowerCase().indexOf(inputValue.toLowerCase()) !== -1;
};
var Select = /** @class */ (function (_super) {
    __extends(Select, _super);
    function Select(props) {
        var _this = _super.call(this, props) || this;
        _this.getLabelFromOption = function (value, children) {
            var label = null;
            React.Children.forEach(children, function (child) {
                if (!child) {
                    return;
                }
                if (child.type.isOptGroup) {
                    var innerLabel = _this.getLabelFromOption(value, child.props.children);
                    if (innerLabel != null) {
                        label = innerLabel;
                    }
                }
                if (value === child.props.value) {
                    label = child.props.children;
                }
            });
            return label;
        };
        _this.onClick = function (e) {
            e.stopPropagation();
            e.nativeEvent.stopImmediatePropagation();
            var disabled = _this.props.disabled;
            if (!disabled) {
                _this.setState({
                    open: !_this.state.open
                });
                document.addEventListener('click', _this.documentOnClick);
            }
        };
        _this.documentOnClick = function () {
            _this.setState({
                open: false
            });
            document.removeEventListener('click', _this.documentOnClick);
            if (_this.inputRef) {
                _this.inputRef.value = '';
            }
        };
        _this.optionClick = function (value) {
            var _a = _this.props, mode = _a.mode, onChange = _a.onChange;
            var selectedValues = _this.state.values;
            if (_this.inputRef) {
                _this.inputRef.blur();
            }
            if (!('value' in _this.props)) {
                var values = selectedValues;
                var exportValue_1 = null;
                if (!mode) {
                    values = [value];
                    exportValue_1 = value.value;
                }
                else {
                    var valueIndex = _this.valuesIndexOf(value);
                    if (valueIndex !== -1) {
                        selectedValues.splice(valueIndex, 1);
                        values = Object.assign([], selectedValues);
                    }
                    else {
                        values = selectedValues.concat(value);
                    }
                    exportValue_1 = [];
                    values.map(function (v) {
                        exportValue_1.push(v.value);
                    });
                }
                _this.setState({
                    values: values,
                    filterWord: null,
                    open: !!mode
                });
                if (onChange) {
                    onChange(exportValue_1);
                }
            }
        };
        _this.valuesIndexOf = function (value) {
            var values = _this.state.values;
            for (var i = 0, len = values.length; i < len; i++) {
                if (values[i].value === value.value) {
                    return i;
                }
            }
            return -1;
        };
        _this.removeValue = function (value) {
            var values = _this.state.values;
            var index = _this.valuesIndexOf(value);
            if (index !== -1) {
                values.splice(index, 1);
                _this.setState({
                    values: Object.assign([], values)
                });
            }
        };
        _this.searchInputOnChange = function (e) {
            _this.setState({
                filterWord: e.target.value
            });
        };
        _this.renderValues = function () {
            var mode = _this.props.mode;
            var _a = _this.state, values = _a.values, filterWord = _a.filterWord;
            if (!mode && values.length > 0) {
                return (React.createElement("div", { className: clsPrefix + '-selected-value' }, filterWord ? null : values[0].label));
            }
            else if (values.length > 0) {
                return (React.createElement("ul", { className: clsPrefix + '-tags' }, values.map(function (item) { return (React.createElement("li", { key: item.value, className: clsPrefix + '-tag' },
                    React.createElement("div", { className: clsPrefix + '-tag-content' }, item.label),
                    React.createElement("span", { className: clsPrefix + '-tag-close', onClick: function (e) {
                            e.stopPropagation();
                            _this.removeValue(item);
                        } },
                        React.createElement(icon_1.default, { type: 'close' })))); })));
            }
            return null;
        };
        _this.renderOptions = function (children) {
            var filterWord = _this.state.filterWord;
            if (filterWord) {
                var sel_1 = [];
                React.Children.forEach(children, function (child) {
                    if (!child) {
                        return;
                    }
                    if (child.type.isOptGroup) {
                        var innerItems = _this.renderOptions(child.props.children);
                        if (innerItems.length) {
                            sel_1.concat(innerItems);
                        }
                        return;
                    }
                    var label = _this.getLabelFromOption(child.props.value, children);
                    if (_this.props.filter(filterWord, { value: child.props.value, label: label })) {
                        sel_1.push(child);
                    }
                });
                return sel_1;
            }
            return children;
        };
        var value = 'value' in props ? props.value : props.defaultValue || [];
        var wrappedValues = [];
        if (Array.isArray(value)) {
            value.map(function (item) {
                wrappedValues.push({
                    value: item,
                    label: _this.getLabelFromOption(item, props.children)
                });
            });
        }
        else {
            wrappedValues.push({
                value: value,
                label: _this.getLabelFromOption(value, props.children)
            });
        }
        _this.state = {
            open: false,
            values: wrappedValues,
            filterWord: null
        };
        return _this;
    }
    Select.prototype.render = function () {
        var _this = this;
        var _a, _b, _c;
        var _d = this.props, placeholder = _d.placeholder, mode = _d.mode, filterable = _d.filterable, disabled = _d.disabled, style = _d.style, children = _d.children;
        var _e = this.state, open = _e.open, filterWord = _e.filterWord, values = _e.values;
        var classes = classnames_1.default((_a = {},
            _a["" + clsPrefix] = true,
            _a[clsPrefix + "-focus"] = this.state.open,
            _a[clsPrefix + "-open"] = this.state.open,
            _a[clsPrefix + "-disabled"] = disabled,
            _a));
        var selectionClasses = classnames_1.default((_b = {},
            _b[clsPrefix + "-selection"] = true,
            _b[clsPrefix + "-selection-single"] = !mode,
            _b[clsPrefix + "-selection-multiple"] = mode,
            _b));
        var dropdownClasses = classnames_1.default((_c = {},
            _c[clsPrefix + "-dropdown"] = true,
            _c[clsPrefix + "-dropdown-multiple"] = mode,
            _c));
        return (React.createElement("div", { className: classes, style: style },
            React.createElement("div", { className: selectionClasses, onClick: this.onClick },
                (placeholder && values.length === 0) ?
                    React.createElement("div", { className: clsPrefix + '-placeholder', style: { display: filterWord ? 'none' : 'block' } }, placeholder) : null,
                this.renderValues(),
                !mode ? (React.createElement("span", { className: clsPrefix + '-arrow' },
                    React.createElement(icon_1.default, { type: open ? 'arrow-up' : 'arrow-down' }))) : null,
                filterable ? (React.createElement("div", { className: clsPrefix + '-search-wrapper', style: { display: open ? 'block' : 'none' } },
                    React.createElement("input", { ref: function (ref) { return _this.inputRef = ref; }, className: clsPrefix + '-search-input', onChange: this.searchInputOnChange }))) : null),
            React.createElement("div", { ref: 'dropdown', className: dropdownClasses },
                React.createElement("ul", { className: clsPrefix + '-dropdown-list' }, this.renderOptions(children)))));
    };
    Select.defaultProps = {
        filter: defaultFilter
    };
    return Select;
}(React.Component));
exports.default = Select;
