"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var classnames_1 = require("classnames");
var Control_1 = require("./Control");
var utils_1 = require("./utils");
var clsPrefix = 'next-slider';
var Slider = /** @class */ (function (_super) {
    __extends(Slider, _super);
    function Slider(props) {
        var _this = _super.call(this, props) || this;
        _this.handleTrackClick = function (e) {
            e.preventDefault();
            var _a = _this.props, max = _a.max, min = _a.min, step = _a.step, range = _a.range, disabled = _a.disabled, vertical = _a.vertical;
            var value = _this.state.value;
            if (disabled) {
                return;
            }
            var sliderDOM = ReactDOM.findDOMNode(_this.refs['slider']);
            var v;
            if (vertical) {
                v = utils_1.calculateY(e, sliderDOM, max, min, step);
            }
            else {
                v = utils_1.calculateX(e, sliderDOM, max, min, step);
            }
            var newValue;
            if (range) {
                if (value[0] > v) {
                    newValue = [v, value[1]];
                }
                else if (value[1] < v) {
                    newValue = [value[0], v];
                }
                else if (v - value[0] > value[1] - v) {
                    newValue = [value[0], v];
                }
                else {
                    newValue = [v, value[1]];
                }
            }
            else {
                newValue = [v];
            }
            _this.setValue(newValue);
        };
        _this.handleFirstControlChange = function (v) {
            var disabled = _this.props.disabled;
            if (disabled) {
                return;
            }
            var value = _this.state.value;
            value[0] = v;
            _this.setValue(value);
        };
        _this.handleSecondControlChange = function (v) {
            var disabled = _this.props.disabled;
            if (disabled) {
                return;
            }
            var value = _this.state.value;
            value[1] = v;
            _this.setValue(value);
        };
        _this.setValue = function (value) {
            var _a = _this.props, onChange = _a.onChange, range = _a.range;
            var sliderValue = value;
            if (!('value' in _this.props)) {
                _this.setState({
                    value: value
                });
            }
            else {
                sliderValue = range ? _this.props.value : [_this.props.value];
            }
            if (onChange) {
                range ? onChange(sliderValue) : onChange(sliderValue[0]);
            }
        };
        var propValue = 'value' in props && props.value ? props.value : props.defaultValue;
        var value = props.range ? propValue : [propValue];
        _this.state = {
            value: value,
        };
        return _this;
    }
    Slider.prototype.componentWillReceiveProps = function (nextProps) {
        if ('value' in nextProps) {
            this.setState({
                value: nextProps.range ? nextProps.value : [nextProps.value]
            });
        }
    };
    Slider.prototype.render = function () {
        var _a;
        var _b = this.props, disabled = _b.disabled, max = _b.max, min = _b.min, step = _b.step, range = _b.range, vertical = _b.vertical;
        var _c = this.state.value, value = _c === void 0 ? [0] : _c;
        var classes = classnames_1.default(clsPrefix, (_a = {},
            _a[clsPrefix + "-vertical"] = vertical,
            _a[clsPrefix + "-disabled"] = disabled,
            _a));
        var barStyle = null;
        if (range) {
            var minValue = void 0, maxValue = void 0;
            if (value[0] > value[1]) {
                minValue = value[1];
                maxValue = value[0];
            }
            else {
                minValue = value[0];
                maxValue = value[1];
            }
            // @ts-ignore
            var leftRatio = Number(minValue / (max - min)).toFixed(2) * 100;
            // @ts-ignore
            var widthRatio = Number((maxValue - minValue) / (max - min)).toFixed(2) * 100;
            if (vertical) {
                barStyle = {
                    top: leftRatio + '%',
                    height: widthRatio + '%'
                };
            }
            else {
                barStyle = {
                    left: leftRatio + '%',
                    width: widthRatio + '%'
                };
            }
        }
        else {
            // @ts-ignore
            var widthRatio = Number(value[0] / (max - min)).toFixed(2) * 100;
            if (vertical) {
                barStyle = {
                    top: 0,
                    height: widthRatio + '%'
                };
            }
            else {
                barStyle = {
                    left: 0,
                    width: widthRatio + '%'
                };
            }
        }
        return (React.createElement("div", { className: classes, onClick: this.handleTrackClick, ref: "slider" },
            React.createElement("div", { className: clsPrefix + '-track' },
                React.createElement("div", { className: clsPrefix + '-bar', style: barStyle }),
                React.createElement(Control_1.default, { value: value[0] || min, clsPrefix: clsPrefix, slider: this, vertical: vertical, disabled: disabled, min: min, max: max, step: step, onChange: this.handleFirstControlChange }),
                range ? (React.createElement(Control_1.default, { value: value[1] || max, clsPrefix: clsPrefix, slider: this, disabled: disabled, vertical: vertical, min: min, max: max, step: step, onChange: this.handleSecondControlChange })) : null)));
    };
    Slider.defaultProps = {
        range: false,
        min: 0,
        max: 100,
        step: 1
    };
    return Slider;
}(React.Component));
exports.default = Slider;
