"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var contains_1 = require("../utils/dom/contains");
var makeChainedFunction_1 = require("../utils/makeChainedFunction");
var Popup_1 = require("./Popup");
var clsPrefix = 'next-tooltip';
function isOneOf(one, of) {
    if (Array.isArray(of)) {
        return of.indexOf(one) >= 0;
    }
    return one === of;
}
var Tooltip = /** @class */ (function (_super) {
    __extends(Tooltip, _super);
    function Tooltip(props) {
        var _this = _super.call(this, props) || this;
        _this.getContainer = function () {
            var container = document.createElement('div');
            container.style.position = 'absolute';
            container.style.top = '0';
            container.style.left = '0';
            container.style.width = '100%';
            document.body.appendChild(container);
            return container;
        };
        _this.makePopup = function (props) {
            var title = props.title, placement = props.placement;
            return (React.createElement(Popup_1.default, { classPrefix: clsPrefix, placement: placement, visible: _this.state.popupVisible, target: _this, onMouseLeave: _this.onMouseLeave, onMouseEnter: _this.onMouseEnter }, title));
        };
        _this.onMouseEnter = function (e) {
            e.preventDefault();
            var popupVisible = _this.state.popupVisible;
            if (!popupVisible) {
                _this.handleVisible(true);
            }
        };
        _this.onMouseLeave = function (e) {
            e.preventDefault();
            var related = e.relatedTarget || e.nativeEvent.toElement;
            if (related && _this.__popup && contains_1.default(_this.__popup.getDOMNode(), related)) {
                return;
            }
            _this.handleVisible(false);
        };
        _this.handleClick = function () {
            _this.handleVisible(!_this.state.popupVisible);
        };
        _this.handleVisible = function (visible) {
            var _a = _this.props, _b = _a.delay, delay = _b === void 0 ? 0.5 : _b, delayShow = _a.delayShow, delayHide = _a.delayHide;
            var millisecond = visible ? (delayShow || delay) * 1000 : (delayHide || delay) * 1000;
            if (millisecond) {
                _this.clearDelayTimer();
                _this.__delayTimer = setTimeout(function () {
                    _this.setVisible(visible);
                    _this.clearDelayTimer();
                }, millisecond);
            }
            else {
                _this.setVisible(visible);
            }
        };
        _this.setVisible = function (visible) {
            if (!('visible' in _this.props)) {
                _this.setState({
                    popupVisible: visible
                });
            }
        };
        _this.clearDelayTimer = function () {
            if (_this.__delayTimer) {
                clearTimeout(_this.__delayTimer);
                _this.__delayTimer = null;
            }
        };
        var popupVisible = props.visible || false;
        _this.state = {
            popupVisible: popupVisible,
            offsetLeft: 0,
            offsetTop: 0
        };
        _this.__delayTimer = null;
        return _this;
    }
    Tooltip.prototype.componentDidMount = function () {
        this.componentDidUpdate();
    };
    Tooltip.prototype.componentWillReceiveProps = function (nextProps) {
        if ('visible' in nextProps) {
            this.setState({
                popupVisible: nextProps.visible || false
            });
        }
    };
    Tooltip.prototype.componentDidUpdate = function () {
        var that = this;
        var props = this.props;
        if (!this.__container) {
            this.__container = this.getContainer();
        }
        ReactDOM.unstable_renderSubtreeIntoContainer(this, this.makePopup(props), this.__container, function () {
            that.__popup = this;
        });
    };
    Tooltip.prototype.componentWillUnmount = function () {
        if (this.__container) {
            ReactDOM.unmountComponentAtNode(this.__container);
            this.__container = null;
            this.__popup = null;
        }
    };
    Tooltip.prototype.render = function () {
        var _this = this;
        var trigger = this.props.trigger;
        var children = this.props.children || React.createElement("span", null);
        var child = React.Children.only(children);
        var childProps = __assign({}, child.props);
        if (isOneOf('click', trigger)) {
            childProps.onClick = makeChainedFunction_1.default(childProps.onClick, this.handleClick);
            childProps.onBlur = makeChainedFunction_1.default(childProps.onBlur, function () { return _this.handleVisible(false); });
        }
        if (isOneOf('hover', trigger)) {
            childProps.onMouseEnter = makeChainedFunction_1.default(childProps.onMouseEnter, this.onMouseEnter);
            childProps.onMouseLeave = makeChainedFunction_1.default(childProps.onMouseLeave, this.onMouseLeave);
        }
        if (isOneOf('focus', trigger)) {
            childProps.onFocus = makeChainedFunction_1.default(childProps.onFocus, function () { return _this.handleVisible(true); });
            childProps.onBlur = makeChainedFunction_1.default(childProps.onBlur, function () { return _this.handleVisible(false); });
        }
        return React.cloneElement(child, childProps);
    };
    Tooltip.defaultProps = {
        placement: 'top',
        trigger: 'hover',
        delay: 0.5
    };
    return Tooltip;
}(React.Component));
exports.default = Tooltip;
